/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.model;

import com.vividsolutions.jts.geom.Coordinate;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Objects;

/**
 * Represents a measurement for a observation.
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class Measurement implements Serializable {

    private Double value;
    public static final String PROP_VALUE = "value";
    private transient PropertyChangeSupport propertyChangeSupport;
    public static final String MIME_TYPE = "application/x-agrosense-measurement";
    private Coordinate coordinate;
    public static final String PROP_COORDINATE = "coordinate";

    /**
     * Get the value
     *
     * @return
     */
    public Double getValue() {
        return value;
    }

    /**
     * Set the value
     *
     * @param value
     */
    public void setValue(Double value) {
        Double old = this.value;
        this.value = value;
        getPropertyChangeSupport().firePropertyChange(PROP_VALUE, old, value);
    }

    /**
     * Get the coordinate for this Measurement
     *
     * @return
     */
    public Coordinate getCoordinate() {
        return coordinate;
    }

    /**
     * Set the coordinate for this measurement
     *
     * @param coordinate
     */
    public void setCoordinate(Coordinate coordinate) {
        Coordinate old = this.coordinate;
        this.coordinate = coordinate;
        getPropertyChangeSupport().firePropertyChange(PROP_COORDINATE, old, coordinate);
    }

    /**
     * Get the property change support
     *
     * @return
     */
    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    /**
     * Add a property change listener to this Measurement
     *
     * @param changeListener
     */
    public void addPropertyChangeListener(PropertyChangeListener changeListener) {
        getPropertyChangeSupport().addPropertyChangeListener(changeListener);
    }

    /**
     * Add a property change listener to this Measurement which listens to the
     * specified property
     *
     * @param prop
     * @param changeListener
     */
    public void addPropertyChangeListener(String prop, PropertyChangeListener changeListener) {
        getPropertyChangeSupport().addPropertyChangeListener(prop, changeListener);
    }

    /**
     * Remove a property change listener from this Measurement
     *
     * @param changeListener
     */
    public void removePropertyChangeListener(PropertyChangeListener changeListener) {
        getPropertyChangeSupport().removePropertyChangeListener(changeListener);
    }

    /**
     * Remove a property change listener from this measurement which was
     * listening to the specified property
     *
     * @param prop
     * @param changeListener
     */
    public void removePropertyChangeListener(String prop, PropertyChangeListener changeListener) {
        getPropertyChangeSupport().removePropertyChangeListener(prop, changeListener);
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 61 * hash + Objects.hashCode(this.value);
        hash = 61 * hash + Objects.hashCode(this.coordinate);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Measurement other = (Measurement) obj;
        if (!Objects.equals(this.value, other.value)) {
            return false;
        }
        if (!Objects.equals(this.coordinate, other.coordinate)) {
            return false;
        }
        return true;
    }
}
