/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.model;

import eu.agrosense.client.grid.Grid;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Describes a collection of {@link Measurement} of a certain
 * {@link MeasurementType}
 *
 * @author Merijn Zengers
 */
public class MeasurementCollection implements Serializable {

    private transient PropertyChangeSupport propertyChangeSupport;
    private List<Measurement> measurements = new ArrayList<>();
    public static final String PROP_MEASUREMENTS = "measurements";
    private final MeasurementType measurementType;
    private transient Grid grid;

    /**
     * Creates a new MeasurementCollection with a {@link MeasurementType} set
     *
     * @param measurementType
     */
    public MeasurementCollection(MeasurementType measurementType) {
        this.measurementType = measurementType;
    }

    /**
     * Get the {@link Measurement}'s this collection holds
     *
     * @return
     */
    public List<Measurement> getMeasurements() {
        return Collections.unmodifiableList(measurements);
    }

    /**
     * Add a {@link Measurement} to the collection of measurements. Fires a
     * {@link PropertyChangeEvent} with {@link PropertyChangeEvent#propertyName}
     * set to {@link MeasurementCollection#PROP_MEASUREMENTS}
     *
     * @param measurement
     */
    public void addMeasurement(Measurement measurement) {
        measurements.add(measurement);
        getPropertyChangeSupport().firePropertyChange(PROP_MEASUREMENTS, null, measurement);
    }

    /**
     * Remove a {@link Measurement} from the collections of measurements. Fires
     * a {@link PropertyChangeEvent} with
     * {@link PropertyChangeEvent#propertyName} set to
     * {@link MeasurementCollection#PROP_MEASUREMENTS}
     *
     * @param measurement
     */
    public void removeMeasurement(Measurement measurement) {
        measurements.remove(measurement);
        getPropertyChangeSupport().firePropertyChange(PROP_MEASUREMENTS, measurement, null);
    }

    /**
     * Get the {@link MeasurementType} of this collection
     *
     * @return
     */
    public MeasurementType getMeasurementType() {
        return measurementType;
    }

    /**
     * Get the grid for this collection Take notice! the grid is transient so
     * not serialized So can be null if not set
     *
     * @return
     */
    public Grid getGrid() {
        return grid;
    }

    /**
     * Set the grid for this collection
     *
     * @param grid
     */
    public void setGrid(Grid grid) {
        this.grid = grid;
    }

    /**
     * Get the property change support
     *
     * @return
     */
    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    /**
     * Add a property change listener to this Measurement
     *
     * @param changeListener
     */
    public void addPropertyChangeListener(PropertyChangeListener changeListener) {
        getPropertyChangeSupport().addPropertyChangeListener(changeListener);
    }

    /**
     * Add a property change listener to this Measurement which listens to the
     * specified property
     *
     * @param prop
     * @param changeListener
     */
    public void addPropertyChangeListener(String prop, PropertyChangeListener changeListener) {
        getPropertyChangeSupport().addPropertyChangeListener(prop, changeListener);
    }

    /**
     * Remove a property change listener from this Measurement
     *
     * @param changeListener
     */
    public void removePropertyChangeListener(PropertyChangeListener changeListener) {
        getPropertyChangeSupport().removePropertyChangeListener(changeListener);
    }

    /**
     * Remove a property change listener from this measurement which was
     * listening to the specified property
     *
     * @param prop
     * @param changeListener
     */
    public void removePropertyChangeListener(String prop, PropertyChangeListener changeListener) {
        getPropertyChangeSupport().removePropertyChangeListener(prop, changeListener);
    }
}
