/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.model;

import com.vividsolutions.jts.geom.Geometry;
import eu.agrosense.client.grid.Grid;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.*;
import nl.cloudfarming.client.model.AbstractEntity;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.apache.commons.collections.list.UnmodifiableList;

/**
 * An observation is a something measured or observed on for instance a field or
 * a crop. An observation can be one observation or a collection of measurements
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class Observation extends AbstractEntity implements Serializable {

    public static final String EXT = "obs";
    //The name of the soil sample
    private String name;
    public static final String PROP_NAME = "name";
    //The date the sample is taken
    private Date sampleDate;
    public static final String PROP_SAMPLE_DATE = "sampleDate";
    private AgroURI source;
    public static final String PROP_SOURCE = "source";
    private Geometry sourceGeometry;
    public static final String PROP_GEOMETRY = "sourceGeometry";
    private List<MeasurementCollection> measurementCollections = new ArrayList<>();
    public static final String PROP_MEASUREMENT_COLLECTIONS = "measurementCollection";

    /**
     * Construct a new observation
     *
     * @param eip
     */
    public Observation(EnterpriseIdProvider eip) {
        super(eip, ItemIdType.OBS);
    }

    /**
     * Construct a new observation with the source set
     *
     * @param eip
     * @param source
     */
    public Observation(EnterpriseIdProvider eip, AgroURI source) {
        super(eip, ItemIdType.OBS);
        this.source = source;
    }

    /**
     * get the name of this soil sample
     *
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of this soil sample
     *
     * @param name
     */
    public void setName(String name) {
        String old = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, old, name);
    }

    /**
     * Get the date the sample is taken
     *
     * @return
     */
    public Date getSampleDate() {
        return sampleDate;
    }

    /**
     * Set the date the sample is taken
     *
     * @param sampleDate
     */
    public void setSampleDate(Date sampleDate) {
        Date old = this.sampleDate;
        this.sampleDate = sampleDate;
        getPropertyChangeSupport().firePropertyChange(PROP_SAMPLE_DATE, old, sampleDate);
    }

    /**
     * Get the {@link AgroURI} of the source for this observation For instance a
     * field
     *
     * @return
     */
    public AgroURI getSource() {
        return source;
    }

    /**
     * Set the {@link AgroURI} of the source
     *
     * @param source
     */
    public void setSource(AgroURI source) {
        this.source = source;
    }

    /**
     * Get the geometry the source had when this observation was created
     *
     * @return
     */
    public Geometry getSourceGeometry() {
        return sourceGeometry;
    }

    /**
     * Set the geometry of the source this observation is performed on
     *
     * @param sourceGeometry
     */
    public void setSourceGeometry(Geometry sourceGeometry) {
        Geometry old = this.sourceGeometry;
        this.sourceGeometry = sourceGeometry;
        getPropertyChangeSupport().firePropertyChange(PROP_GEOMETRY, old, sourceGeometry);
    }

    /**
     * Add a Measurement collection to this observation. Fires a
     * {@link PropertyChangeEvent} with {@link PropertyChangeEvent#propertyName}
     * set to {@link Observation#PROP_MEASUREMENT_COLLECTIONS}
     *
     * @param collection
     */
    public void addMeasurementCollection(MeasurementCollection collection) {
        measurementCollections.add(collection);
        getPropertyChangeSupport().firePropertyChange(PROP_MEASUREMENT_COLLECTIONS, null, collection);
    }

    /**
     * Remove a Measurement collection from this observation.
     * Fires a
     * {@link PropertyChangeEvent} with {@link PropertyChangeEvent#propertyName}
     * set to {@link Observation#PROP_MEASUREMENT_COLLECTIONS}
     * 
     * @param collection 
     */
    public void removeMeasurementCollection(MeasurementCollection collection) {
        measurementCollections.remove(collection);
        getPropertyChangeSupport().firePropertyChange(PROP_MEASUREMENT_COLLECTIONS, collection, null);
    }

    /**
     * Get the list of {@link MeasurementCollection}'s this observation holds
     * @return an {@link UnmodifiableList} 
     */
    public List<MeasurementCollection> getMeasurementCollections() {
        return Collections.unmodifiableList(measurementCollections);
    }
}
